#include "asteroids.h"
#include <stdio.h>

pixel background;
pixel ship_c;
pixel bullet_c;
pixel ast_c;

obj bullets[MAX_BULLETS];
ast asteroids[MAX_ASTEROIDS];

obj ship;

int cooldown;

int score;
int frames;

int frame_delay;
int frame_delay_sub;

int lifes;

int high_score;

int ast_speed = 1;

int level = 1;

int max(int x, int y) {
	return (x > y ? x : y);
}

int min(int x, int y) {
	return (x < y ? x : y);
}

int abs(int a) {
	return (a < 0 ? -a : a);
}

void init_asteroids() {
	background.r = 6;
	background.g = 6;
	background.b = 6;

	ship_c.r = 130;
	ship_c.g = 20;
	ship_c.b = 30;

	bullet_c.r = 200;
	bullet_c.g = 200;
	bullet_c.b = 200;

	ast_c.r = 110;
	ast_c.g = 110;
	ast_c.b = 110;

	ship.p.x = START_X;
	ship.p.y = START_Y;
	ship.radius = SHIP_RADIUS;

	score = 0;
	frames = 0;

	for (int i = 0; i < MAX_BULLETS; i++) {
		bullets[i].radius = -1;
	}

	for (int i = 0; i < MAX_ASTEROIDS; i++) {
		asteroids[i].radius = -1;
	}

	cooldown = 0;

	ast_speed = 2;
	level = 1;

	lifes = INIT_LIFES;

	ST7735R_FillRect(0, 0, ST7735R_WIDTH - 1, ST7735R_HEIGHT - 1, background.r, background.g, background.b);
}

void lvl_up() {
	if (level == 1) {
		if (score < LVL2_SCORE)
			return;

		level = 2;

		ship_c.r = 150;
		ship_c.g = 100;
		ship_c.b = 20;

		if (lifes < INIT_LIFES) {
			lifes ++;
		}

		erase_obj(ship);

		ast_speed = 3;
		ship.radius = SHIP_RADIUS - 1;

		draw_obj(ship, ship_c);
	}
	if (level == 2) {
		if (score < LVL3_SCORE)
			return;

		level = 3;

		ship_c.r = 40;
		ship_c.g = 0;
		ship_c.b = 100;

		if (lifes < INIT_LIFES) {
			lifes ++;
		}

		erase_obj(ship);

		ast_speed = 4;
		ship.radius = SHIP_RADIUS - 2;

		draw_obj(ship, ship_c);
	}
}

int update() {
	int r;

	lvl_up();
	fire(ship);
	generate_asteroids();

	frames++;

	if (frames >= FRAMES_PER_SCORE) {
		score += level;
		frames = 0;
	}

	for (int i = 0; i < MAX_BULLETS; i++) {
		if (bullets[i].radius == -1)
			continue;

		erase_obj(bullets[i]);
		bullets[i].p.y -= BULLET_SPEED;

		for (int j = 0; j < MAX_ASTEROIDS; j++) {
			if (asteroids[i].radius == -1)
				continue;

			r = collide_bullet(asteroids[j], bullets[i]);
			
			if (r == 1) {
				erase_ast(asteroids[j]);
				bullets[i].radius = -1;
				asteroids[j].hp --;
				asteroids[j].radius = asteroids[j].hp * 2;
				draw_ast(asteroids[j]);
				break;
			}
			if (r == 2) {
				erase_ast(asteroids[j]);
				asteroids[j].radius = -1;
				bullets[i].radius = -1;
				score += asteroids[j].score;
				break;
			}
		}

		if (bullets[i].radius == -1)
			continue;

		if (bullets[i].p.y <= 0) {
			bullets[i].radius = -1;
		} else {
			draw_obj(bullets[i], bullet_c);
		}
	}

	for (int i = 0; i < MAX_ASTEROIDS; i++) {
		if (asteroids[i].radius == -1)
			continue;

		erase_ast(asteroids[i]);
		asteroids[i].p.y += ast_speed;

		if (collide(ship, asteroids[i]) == 1) {
			lifes--;

			if (lifes <= 0) {
				int local_score = score;
				init_asteroids();
				print_high_scores(local_score);
				return -1;
			}

			ST7735R_FillRect(0, 0, ST7735R_WIDTH - 1, ST7735R_HEIGHT - 1, background.r, background.g, background.b);

			ship.p.x = START_X;
			ship.p.y = START_Y;

			for (int k = 0; k < MAX_BULLETS; k++) {
				bullets[k].radius = -1;
			}

			for (int k = 0; k < MAX_ASTEROIDS; k++) {
				asteroids[k].radius = -1;
			}

			break;
		}

		if (asteroids[i].p.y >= ST7735R_HEIGHT) {
			asteroids[i].radius = -1;
		} else {
			draw_ast(asteroids[i]);
		}
	}

	draw_score();
	draw_lifes();

	return 0;
}

void move_ship(float dx, float dy) {
	erase_obj(ship);

	if (dx > TOL1 && ship.p.x > SHIP_RADIUS + 2)
		ship.p.x -= 1;
	if (dx > TOL2 && ship.p.x > SHIP_RADIUS + 2)
		ship.p.x -= 1;
	if (dx > TOL3 && ship.p.x > SHIP_RADIUS + 3)
		ship.p.x -= 2;
	if (dx > TOL4 && ship.p.x > SHIP_RADIUS + 3)
		ship.p.x -= 2;

	if (dx < -TOL1d && ship.p.x < ST7735R_WIDTH - SHIP_RADIUS - 5)
		ship.p.x += 1;
	if (dx < -TOL2d && ship.p.x < ST7735R_WIDTH - SHIP_RADIUS - 5)
		ship.p.x += 1;
	if (dx < -TOL3d && ship.p.x < ST7735R_WIDTH - SHIP_RADIUS - 6)
		ship.p.x += 2;
	if (dx < -TOL4d && ship.p.x < ST7735R_WIDTH - SHIP_RADIUS - 6)
		ship.p.x += 2;

	if (dy > TOL / 2 && ship.p.y > HIGH_Y) {
		ship.p.y -= 1;
		if (dy > TOL)
			ship.p.y -= 1;
	}

	if (dy < -TOL && ship.p.y < LOW_Y) {
		ship.p.y += 1;
		if (dy < -TOL*2)
			ship.p.y += 1;
	}

	draw_obj(ship, ship_c);
}

void draw_obj(obj o, pixel c) {
	ST7735R_FillRect(o.p.x - o.radius, o.p.y - o.radius, o.p.x + o.radius, o.p.y + o.radius, c.r, c.g, c.b);
}

void draw_ast(ast o) {
	ST7735R_FillRect(o.p.x - o.radius, o.p.y - o.radius, o.p.x + o.radius, o.p.y + o.radius, o.c.r, o.c.g, o.c.b);
}

void erase_ast(ast o) {
	ST7735R_FillRect(o.p.x - o.radius, o.p.y - o.radius, o.p.x + o.radius, o.p.y + o.radius, background.r, background.g, background.b);
}

void erase_obj(obj o) {
	ST7735R_FillRect(o.p.x - o.radius, o.p.y - o.radius, o.p.x + o.radius, o.p.y + o.radius, background.r, background.g, background.b);
}

void fire(obj ship) {
	if (cooldown > 0) {
		cooldown--;
		return;
	} else {
		cooldown = COOLDOWN;
	}

	for (int i = 0; i < MAX_BULLETS; i++) {
		if (bullets[i].radius == -1) {
			bullets[i].radius = 0;
			bullets[i].p.x = ship.p.x;
			bullets[i].p.y = ship.p.y - ship.radius;
			break;
		}
	}
}

void generate_asteroids() {
	if (rand() / (RAND_MAX / AST_SPAWN_CHANCE + 1) != 0)
		return;
	
	for (int i = 0; i < MAX_ASTEROIDS; i++) {
		if (asteroids[i].radius == -1) {
			asteroids[i].radius = rand() / (RAND_MAX / (MAX_AST_RADIUS - MIN_AST_RADIUS) + 1) + MIN_AST_RADIUS;
			asteroids[i].p.x = rand() / (RAND_MAX / (ST7735R_WIDTH - asteroids[i].radius * 2) + 1) + asteroids[i].radius;

			asteroids[i].p.y = 0;

			asteroids[i].c.r = 100 + rand() / (RAND_MAX / 20);
			asteroids[i].c.g = 100 + rand() / (RAND_MAX / 20);
			asteroids[i].c.b = 100 + rand() / (RAND_MAX / 20);

			asteroids[i].hp = asteroids[i].radius / 2;
			asteroids[i].score = asteroids[i].hp * 2;

			break;
		}
	}
}

int collide(obj a, ast b) {
	int dx = abs((a.p.x - b.p.x));
	int dy = abs((a.p.y - b.p.y));

	if (max(dx, dy) < a.radius + b.radius)
		return 1;

	return 0;
}

int collide_bullet(ast a, obj bullet) {
	int dx = abs((a.p.x - bullet.p.x));
	int dy;

	for (int i = 0; i < BULLET_SPEED; i++) {
		dy = abs((a.p.y - bullet.p.y + i));

		if (max(dx, dy) <= a.radius) {
			a.hp--;
			if (a.hp <= 0) {
				return 2;
			}

			return 1;
		}
	}

	return 0;
}

void draw_score() {
	char buffer[20];

	sprintf(buffer,"Score:%d", score);

	ST7735R_DrawText(0, 1, buffer, 255, 255, 255, background.r, background.g, background.b);
}

void print_high_scores(int score) {
	int high_scores[HIGH_SCORES];
	int i, j;
	int aux;
	char buffer[20];

	for (i = 0; i < HIGH_SCORES; i ++) {
		high_scores[i] = eeprom_read_dword((const uint32_t *) (i * 4) );
	}

	for (i = 0; i < HIGH_SCORES; i++)
		for (j = i; j < HIGH_SCORES; j++)
			if (high_scores[i] < high_scores[j]) {
				aux = high_scores[i];
				high_scores[i] = high_scores[j];
				high_scores[j] = aux;
			}

	for (i = 0; i < HIGH_SCORES; i++) {
		if (score > high_scores[i]) {
			for (j = HIGH_SCORES - 1; j > i; j--) {
				high_scores[j] = high_scores[j - 1];
			}
			high_scores[i] = score;
			break;
		}
	}

	for (i = 0; i < HIGH_SCORES; i ++) {
		eeprom_update_dword((uint32_t *) (i * 4), (uint32_t)high_scores[i]);
	}

	ST7735R_FillRect(0, 0, ST7735R_WIDTH - 1, ST7735R_HEIGHT - 1, background.r, background.g, background.b);

	j = 10;
	if (score > 0) {
		sprintf(buffer,"Your score: %d", score);
		ST7735R_DrawText(10, j, buffer, 255, 255, 255, background.r, background.g, background.b);
	}

	j += 20;
	sprintf(buffer,"Top 10 scores:");
	ST7735R_DrawText(30, j, buffer, 255, 255, 255, background.r, background.g, background.b);

	j += 2;

	for (i = 0; i < HIGH_SCORES; i++) {
		j += 8;
		sprintf(buffer,"%d", high_scores[i]);
		ST7735R_DrawText(50, j, buffer, 255, 255, 255, background.r, background.g, background.b);
	}

	_delay_ms(4000);

	ST7735R_FillRect(0, 0, ST7735R_WIDTH - 1, ST7735R_HEIGHT - 1, background.r, background.g, background.b);
}

void draw_lifes() {
	char buffer[15];

	switch (lifes) {
		case 1:
			sprintf(buffer,"    <3");
			break;
		case 2:
			sprintf(buffer,"  <3<3");
			break;
		case 3:
			sprintf(buffer,"<3<3<3");
			break;
	}

	ST7735R_DrawText(90, 1, buffer, 255, 255, 255, background.r, background.g, background.b);
}
