#ifndef ASTEROIDS_H_
#define ASTEROIDS_H_

#include "ST7735R_TFT.h"
#include "MPU6050_res_define.h"
#include "I2C_Master_H_file.h"
#include <time.h>
#include <avr/eeprom.h>

#define SHIP_RADIUS 4
#define START_X 60
#define START_Y 150

#define LOW_Y 150
#define HIGH_Y 100

#define TOL 0.1f

#define TOL1 0.05f
#define TOL2 0.2f
#define TOL3 0.35f
#define TOL4 0.5f

#define TOL1d 0.03f
#define TOL2d 0.12f
#define TOL3d 0.21f
#define TOL4d 0.3f

#define MAX_BULLETS 20
#define MAX_ASTEROIDS 7
#define MIN_AST_RADIUS 2
#define MAX_AST_RADIUS 7
#define COOLDOWN 8
#define AST_SPAWN_CHANCE 10

#define FRAMES_PER_SCORE 40

#define BULLET_SPEED 3

#define MAX_FRAME_DELAY 6
#define FRAME_DELAY_SUB_SCORE 30

#define INIT_LIFES 3

#define HIGH_SCORES 10

#define LVL2_SCORE 150
#define LVL3_SCORE 600

typedef struct pixel {
	int r;
	int g;
	int b;
} pixel;

typedef struct pos {
	int x;
	int y;
} pos;

typedef struct obj {
	pos p;
	int radius;
} obj;

typedef struct ast {
	pos p;
	int radius;
	int hp;
	int score;
	pixel c;
} ast;

void init_asteroids();

int update();

void move_ship(float dx, float dy);

void draw_obj(obj o, pixel c);

void draw_ast(ast a);

void erase_ast(ast o);

void erase_obj(obj o);

void fire(obj ship);

void generate_asteroids();

int collide(obj a, ast b);

int collide_bullet(ast a, obj bullet);

void draw_score();

void print_high_scores(int score);

void draw_lifes();

void restart();

void lvl_up();

#endif