#define F_CPU 16000000
#include <avr/io.h>
#include <stdio.h>
#include <stdint.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "spi.h"
#include "ST7735R_TFT.h"
#include "MPU6050_res_define.h"
#include "I2C_Master_H_file.h"
#include <avr/eeprom.h>

#include "asteroids.h"

#define SAMPLE_COUNT 20

float Acc_x,Acc_y,Acc_z,Temperature,Gyro_x,Gyro_y,Gyro_z;

void MPU6050_Init()										/* Gyro initialization function */
{
	_delay_ms(150);										/* Power up time >100ms */
	I2C_Start_Wait(0xD0);								/* Start with device write address */
	I2C_Write(SMPLRT_DIV);								/* Write to sample rate register */
	I2C_Write(0x07);									/* 1KHz sample rate */
	I2C_Stop();

	I2C_Start_Wait(0xD0);
	I2C_Write(PWR_MGMT_1);								/* Write to power management register */
	I2C_Write(0x01);									/* X axis gyroscope reference frequency */
	I2C_Stop();

	I2C_Start_Wait(0xD0);
	I2C_Write(CONFIG);									/* Write to Configuration register */
	I2C_Write(0x00);									/* Fs = 8KHz */
	I2C_Stop();

	I2C_Start_Wait(0xD0);
	I2C_Write(GYRO_CONFIG);								/* Write to Gyro configuration register */
	I2C_Write(0x18);									/* Full scale range +/- 2000 degree/C */
	I2C_Stop();

	I2C_Start_Wait(0xD0);
	I2C_Write(INT_ENABLE);								/* Write to interrupt enable register */
	I2C_Write(0x01);
	I2C_Stop();
}

void MPU_Start_Loc()
{
	I2C_Start_Wait(0xD0);								/* I2C start with device write address */
	I2C_Write(ACCEL_XOUT_H);							/* Write start location address from where to read */ 
	I2C_Repeated_Start(0xD1);							/* I2C start with device read address */
}

void Read_RawValue()
{
	MPU_Start_Loc();									/* Read Gyro values */
	Acc_x = (((int)I2C_Read_Ack()<<8) | (int)I2C_Read_Ack());
	Acc_y = (((int)I2C_Read_Ack()<<8) | (int)I2C_Read_Ack());
	Acc_z = (((int)I2C_Read_Ack()<<8) | (int)I2C_Read_Ack());
	// Temperature = (((int)I2C_Read_Ack()<<8) | (int)I2C_Read_Ack());
	// Gyro_x = (((int)I2C_Read_Ack()<<8) | (int)I2C_Read_Ack());
	// Gyro_y = (((int)I2C_Read_Ack()<<8) | (int)I2C_Read_Ack());
	// Gyro_z = (((int)I2C_Read_Ack()<<8) | (int)I2C_Read_Nack());
	I2C_Stop();
}

void Read_Fine_Values(float *x, float *y, float *z)
{
	int i = 0;

	*x = 0;
	*y = 0;
	*z = 0;

	for (i = 0; i < SAMPLE_COUNT; i++) {
		Read_RawValue();

		*x += Acc_x/16384.0;
		*y += Acc_y/16384.0;
		*z += Acc_z/16384.0;
	}

	*x /= SAMPLE_COUNT;
	*y /= SAMPLE_COUNT;
	*z /= SAMPLE_COUNT;
}

int main(void)
{
	float Xa,Ya,Za;

	/* Initialize I2C */
	I2C_Init();

	/* Initialize MPU6050 */
	MPU6050_Init();

	// Init SPI
	SPI_init();

	// Init LCD
	ST7735R_Begin();

	// Init game.
	init_asteroids();

	print_high_scores(-2);

	for(;;)
	{
		Read_Fine_Values(&Xa, &Ya, &Za);
		
		move_ship(Xa, Ya);
		update();
	}

	return 0;
}